/* The samplerate used, must be a mp3 rate below 48000 to allow rate tuning. */
#define UART_OUTPUT 44100U //sets samplerate
//#define UART_OUTPUT 22050U
#define LINE_IN_ENABLE (1<<6) /*GPIO6 selects mono-mic (0)/stereo line-in (1)*/
#define ENABLE_HIGHREF /* for 1.65V reference voltage */
#define USE_MP3 /* Define to use mp3 encoding, if not set, uses IMA_ADPCM. */
/* The UART bitrate of 460800bps is used. */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define STANDALONE
#define VS1053
#include <hardware.h>
#include <imalib.h>
#include <encoders.h>
#include <parametric.h>
//#include "standalone.h"
void CallEncoders(void); /*patched Encoders()*/

/* Check for and correct channel swap (DECIM restart bug)
   This app always starts from reset, so really does not need this.
 */
void DecimCheck(void) {
#if 0
    register u_int16 decim = USEX(DECIM_CONTROL);
    register u_int16 lim = 0, val;
    register u_int16 tries = 0;
    decim |= DECIM_ENABLE | DECIM_MODU2_PD; /*right chn to powerdown*/
    lim = clockX * (512/2 * 14); /* 13 seems enough, we use 14 */
    if (USEX(SCI_STATUS) & (1<<SCIST_AD_CLOCK))
        lim <<= 1; /*3MHz mode, double the wait*/
 again:
    tries++;
    USEX(DECIM_CONTROL) = decim;
    /* Wait at least 14 samples .. */
    {
        register u_int16 i;
        for (i=0; i<lim; i++) { /*two cycles per loop*/
            USEX(DECIM_DATA_LEFT);
            USEX(DECIM_DATA_LEFT);
        }
    }
    /*if left chn is min/max, wrong order*/
    val = USEX(DECIM_DATA_LEFT);
    if (val == 0x8000U || val == 0x7fffU) {
        if (tries < 0x40) {
            USEX(DECIM_CONTROL) &= ~DECIM_ENABLE;
            goto again;
        }
    }
    /* take the other channel out of powerdown */
    USEX(DECIM_CONTROL) = decim & ~DECIM_MODU2_PD;
#endif
}


//void putch(register __a0 int ch);
void putword(register __a0 int ch);

#if 0 /* not hooked with the current startup module */
/*
  Called before HALT
 */
void UserHook(void) {
}
#endif

void main(void) {
#ifdef ENABLE_HIGHREF
    USEX(SCI_STATUS) |= (1<<SCIST_REFERENCE_SEL); /*1.65V*/
#endif
    /* Make sure we have the right upper ROM visible. */
    USEX(VS1053_IROM4) &= ~(IROM4_ENABLE);
    USEX(SCI_HDAT1) = 0; /* clear so FLAC is not indicated. */

    hwSampleRate = 11; /*force, mem is only cleared at boot, not restart*/
    uartByteSpeed = 9600/10; //initial rate
    InitHardware(); /*no longer clears memory in vs1053/63 so preserves const*/
    audioPtr.curFctl = audioPtr.newFctl = 0x15555; //correct for 8kHz 20111117

    USEX(SCI_STATUS) &= ~((1<<SCIST_APDOWN1) | (1<<SCIST_APDOWN2));
    USEX(INT_ENABLE) = 0;
    /* Allow DREQ and enable interrupts, VERY IMPORTANT! */
    USEX(INT_GLOB_ENA) = 0;
    USEX(INT_GLOB_ENA) = 0;
    USEX(SER_DREQ) = 1;
    clockX = 9; /*4.5x required for 44+k stereo recording */
    SetRate(hwSampleRate);
//    USEX(UART_DIV) = UartDiv();

    USEX(SCI_AICTRL0) = UART_OUTPUT; //<48k to allow rate adjustment
    //USEX(SCI_AICTRL1) = 0x0000;     /* auto gain */
    USEX(SCI_AICTRL1) = 0x0400;     /* fixed 1.0x gain */
    USEX(SCI_AICTRL2) = 0x2000; /* max gain for AGC */

    /* 480600bps @ 4.5x clock */
    parametric_x.i.encoding.txUartDiv = 0x1306;
    parametric_x.i.encoding.txUartByteSpeed = 0;
    parametric_x.i.encoding.txPauseGpio = 0;

    /* GPIO6 is high -> stereo line */
#if defined(LINE_IN_ENABLE) && LINE_IN_ENABLE
    if ((USEX(GPIO_IDATA) & LINE_IN_ENABLE)) {
        /* mode: mp3, joint stereo

           Note: 48kHz sample rate with stereo is too much for
           3.5x clock. Thus we set 4.5x in record mode.

           48kHz should not be used so the receiver can adjust rate easily.
        */
#if UART_OUTPUT < 32000
        USEX(SCI_WRAMADDR) = 0xe000 + 192; //192kbit/sec
#else
        USEX(SCI_WRAMADDR) = 0xe000 + 256; //256kbit/sec
#endif
        USEX(SCI_AICTRL3) = AICTRL3_MP3 | AICTRL3_STEREO | AICTRL3_ENABLEUART;
        USEX(SCI_MODE) |= (1<<SCIMB_LINE); /* Use line input -- no amp. */
    } else
#endif
    {
        /* mode: mp3, mono left channel(mic) */
#ifdef USE_MP3
        USEX(SCI_WRAMADDR) = 0xe000 + 128; //128kbit/sec
        USEX(SCI_AICTRL3) = AICTRL3_MP3 | AICTRL3_LEFT | AICTRL3_ENABLEUART;
#else
        /* IMA ADPCM
           1. the receiver needs to be ready when we start sending,
           2. or the format (and rate, channels) needs to be known in the
              decoder, and encoder use AICTRL3_NORIFF.
        */
        USEX(SCI_AICTRL3) = AICTRL3_IMAADPCM | AICTRL3_LEFT | AICTRL3_ENABLEUART;
#endif
        USEX(SCI_MODE) &= ~(1<<SCIMB_LINE); /* Use mic input */
USEX(SCI_AICTRL1) = 0x0000;     /* auto gain */
USEX(SCI_AICTRL2) = 0x1000; /* max 4x gain for AGC */
//USEX(SCI_AICTRL2) = 0x4000; /* max 16x gain for AGC */
    }
    USEX(SCI_MODE) &= ~(1<<SCIMB_CANCEL);

    /*
      NOTE: Encoders() sets applAddr to NULL, but on the other
      hand it does not get called anyway because encoders
      do not use AudioOutputSamples().

      IdleHook is still called, but we have not hooked that here.
    */
#if 1
    CallEncoders(); /* with patch. loop there until reset */
#else
    CallIROM4(Encoders); /* loop there until reset */
#endif
    while (1)
        ;
}

