#define STANDALONE
#define VS1053
#include <hardware.h>
#include <imalib.h>
#include <encoders.h>
#include <asmfuncs.h>
#include <stream.h>
#include <parametric.h>
#include <stdlib.h>

/*
  There are two modes:
  1.Stream anything up to 44.1kHz
  - If you stream mp3, the decoder will automatically synchronize to the
    data, and determine the number of channels and samplerate from the mp3
    frame headers.
  - If you stream something else than mp3, the decoder needs to receive the
    headers. This means the receiver (this software) needs to be started
    first, and the encoder (UartSender) afterwards.

  2.Stream mono IMA ADPCM up to 44.1kHz (enable the define STREAM_ADPCM below)
  - The decoder will expect mono IMA ADPCM with block align 256 (505 samples
    per frame), synchronizing to the IMA ADPCM headers, and determining
    the samplerate from the time between two IMA ADPCM blocks.
  - If you buffer more than one IMA ADPCM frame to be sent as a burst,
    you should disable samplerate detection.

  The UART baudrate is 460800bps. You can change it from vs1063a-uartin.s .

 */

//#define STREAM_ADPCM //stream mono IMA ADPCM at 44.1kHz


void putch(register __a0 int ch);
void putword(register __a0 int ch);

s_int32 fillFlt = 0;

void AdaptRate(void) {
  s_int16 diff = StreamDiff();
  s_int16 audiff = AudioBufFill()/2;
  //USEX(SCI_AICTRL0) = diff;
  //USEX(SCI_AICTRL1) = audiff;
#define GOOD_FILL (AUDIO_BUFFER_SZ/4) //AUDIO_BUFFER_SZ/3
  fillFlt = fillFlt - (fillFlt >> 7) + (audiff - GOOD_FILL/2) + diff;
  {
    u_int32 fillTmp = audioPtr.curFctl + fillFlt/8;
    static s_int32 oldFlt = 0;
    if (labs(oldFlt - fillTmp) > 2) {
      oldFlt = fillTmp;
      USEX(FREQCTLL) = fillTmp;
      USEX(FREQCTLH) = (USEX(FREQCTLH) & 0xfff0) | ((u_int16)((u_int32)fillTmp>>16) & 15);
      
      //USEX(SCI_AICTRL2) = USEX(FREQCTLL);
      //USEX(SCI_AICTRL3) = USEX(FREQCTLH);
    }
  }
}

auto s_int16 SearchForImaHeader(s_int16 chans);
void ReadStreamY(register __i2 __y s_int16 *p, register __c0 s_int16 words);
auto void MyGetCPairs(register __i0 __y u_int16 *p, register __a0 s_int16 n);
#define PERIP32(x) (*(volatile u_int32 __mem_x *)(u_int32)(x))

u_int32 MyCheckTag(register __reg_d u_int32 head) {
#ifdef STREAM_ADPCM
  static __mem_y s_int16 imaTmpY[4];
  static s_int16 imaTmpX[8], tmpBuf[16];
  s_int16 adpcmPhase = 0;
  s_int16 val = 0;
  s_int16 st = 0;
  USEX(VS1053_IROM4) = IROM4_ENABLE;
  SetHardware(1, 44100U); //Only 1 channel supported
  /* Force the new rate active immediately or AdaptRate() will not work. */
  audioPtr.changeTimer = 1;
  /* IMA ADPCM decoding, implement it here. */
  while (1) {
      if (adpcmPhase == 0) {
        SearchForImaHeader(1); //Mono -- TODO: determine samplerate
        MyGetCPairs((__mem_y void *)imaTmpY, 4/2);
        AdaptRate();
        ImaExpandStart(&st, &val, (__mem_y void *)imaTmpY, (void *)imaTmpX);
        tmpBuf[0] = imaTmpX[0];
        tmpBuf[1] = imaTmpX[0];
        WmaStereoCopy(tmpBuf, 1);
        adpcmPhase += 1;
      } else {
        MyGetCPairs((__mem_y void *)imaTmpY, 4/2);
        ImaExpandContinue(8, &st, &val, (__mem_y void *)imaTmpY, (void *)imaTmpX);
        {
          register s_int16 *d = tmpBuf;
          register s_int16 *s = imaTmpX;
          register int i;
          for (i=0;i<8;i++) {
            *d++ = *s;
            *d++ = *s++;
          }
        }
        WmaStereoCopy(tmpBuf, 8);
        adpcmPhase += 8;
        if (adpcmPhase >= 505) {
          adpcmPhase = 0;
        }
      }
  }
#pragma msg 38 off
#pragma msg 36 off
#else
  AdaptRate();
  return head;
#endif
}

#pragma msg 38 on
#pragma msg 36 on

u_int16 triggerCount = 0;
s_int16 ApplAddr(s_int16 register __i0 **d, s_int16 register __a1 mode,
        	 s_int16 register __a0 n) {
    USEX(INT_ENABLE) = 0x23; //enable DAC, SCI, RX
    //If not mp3, use some adaptation anyway.
    //Less accurate because it doesn't happen on encoding block boundaries.
    if (mode == APPL_AUDIO && USEX(SCI_HDAT1) == 0x7665 /*'ve'*/) {
      triggerCount += n;
      if (triggerCount >= hwSampleRate / 32) {
        triggerCount = 0;
        AdaptRate();//MyCheckTag(0);
      }
    }
    return n;
}
